<!DOCTYPE html>
<html lang="{{ page.lang | default: site.lang | default: "en" }}">

  {%- include head.html -%}

  <body>

    {%- include header.html -%}

    <div class="py-5">
      <div class="container page-content">
          {{ content }}
      </div>
    </div>

    {%- include footer.html -%}

  </body>

</html>
