<head>
 <meta charset="utf-8">
  <meta http-equiv="X-UA-Compatible" content="IE=edge">
  <meta name="viewport" content="width=device-width, initial-scale=1">
  {%- seo -%}
  <link rel="stylesheet" href="{{ "/assets/css/main.css" | relative_url }}">
  {%- feed_meta -%}
  <script src="{{ "/assets/js/bootstrap/jquery.min.js" | relative_url }}"></script>
  <script src="{{ "/assets/js/bootstrap/bootstrap.bundle.min.js" | relative_url }}"></script>
</head>
