---
layout: default
---
<article>

  <header>
    <h1>{{ page.title | escape }}</h1>
    <p class="text-muted">
      <time datetime="{{ page.date | date_to_xmlschema }}">
        {%- assign date_format = "%b %-d, %Y" -%}
        {{ page.date | date: date_format }}
      </time>
      {%- if page.author -%}
        • {{ page.author }}
      {%- endif -%}
    </p>
  </header>

  <section>
    {{ content }}
  </section>

</article>
