<div class="py-1 bg-light">
  <div class="container">
    <div class="row">
      <div class="col-sm text-right">
        <a href="{{ site.github.repository_url }}/tree/master/docs/{{ page.path }}">
          edit this page <i class="far fa-edit"></i>
        </a>
      </div>
    </div>
  </div>
</div>

<!--div class="py-5 border-top">
  <div class="container">

    <p class="h5 mb-3">{{ site.title | escape }}</p>

    <div class="row">

      <div class="col-sm">
        <ul class="list-unstyled">
          <li>
            {%- if site.author -%}
              {{ site.author | escape }}
            {%- else -%}
              {{ site.title | escape }}
            {%- endif -%}
          </li>
          {%- if site.email -%}
            <li>
              <a href="mailto:{{ site.email }}">
                {{ site.email }}
              </a>
            </li>
          {%- endif -%}
        </ul>
      </div>


      <div class="col-sm text-right">
        <p>{{- site.description | escape -}}</p>
      </div>

    </div>

  </div>
</div-->
<script src="{{ "/assets/js/bootstrap-additions.js" | relative_url }}"></script>
<script src="{{ "/assets/js/anchors.js" | relative_url }}"></script>
