---
layout: default
---

<div>

  {%- if page.title -%}
  <header>
    <h1>{{ page.title }}</h1>
  </header>
  {%- endif -%}

  <section>
    {{ content }}
  </section>

  {%- if site.posts.size > 0 -%}
    <p class="h4">{{ page.list_title | default: "Posts" }}</p>
    <ul class="list-unstyled">
      {%- for post in site.posts -%}
      <li>
        <span class="text-muted">
          {%- assign date_format = "%b %-d, %Y" -%}
          {{ post.date | date: date_format }}
        </span>
        <p class="h5">
          <a href="{{ post.url | relative_url }}">
            {{ post.title | escape }}
          </a>
        </p>
        {%- if site.show_excerpts -%}
          {{ post.excerpt }}
        {%- endif -%}
      </li>
      {%- endfor -%}
    </ul>

    <p class="rss-subscribe">subscribe <a href="{{ "/feed.xml" | relative_url }}">via RSS</a></p>
  {%- endif -%}

</div>
