<nav class="navbar navbar-expand-lg navbar-light bg-light">
  <div class="container">
    <a class="navbar-brand"
    href="{{ "/" | relative_url }}">
      <!--img width="30" height="30"
           class="d-inline-block align-top"
           src="{{ "/assets/img/logo.png" | relative_url }}"
           alt="ts-jest Logo"-->
      {{ site.title | escape }}
    </a>

    <button class="navbar-toggler" type="button" data-toggle="collapse" data-target="#navbarSupportedContent"
      aria-controls="navbarSupportedContent" aria-expanded="false" aria-label="Toggle navigation">
      <span class="navbar-toggler-icon"></span>
    </button>
    <div class="collapse navbar-collapse" id="navbarSupportedContent">
      {% include nav.html %}
    </div>
  </div>
</nav>
