p > img[align="left"] {
  margin-right: 0.5em;
}

blockquote {
  > p {
    margin: 0;
  }
  color: $info;
  background-color: $light;
  padding: 0.75rem;
}

h2 {
  margin-top: 1.5rem;
  &::before {
    display: block;
    content: '';
    margin-top: $hr-margin-y;
    margin-bottom: $hr-margin-y;
    border: 0;
    border-top: $hr-border-width solid $hr-border-color;
    }
}
h3 {
  margin-top: 1rem;
}
h4 {
  margin-top: .75rem;
}

div.highlighter-rouge {
  margin-bottom: 1rem;
}
