h2,h3,h4,h5,h6 {
  &[id] {
    position: relative;
    > a.auto-anchor {
      position: absolute;
      left: -2rem;
      // top: 1rem;
      right: 0;
      display: none;
      > i {
        font-size: 1.5rem !important;
      }
    }
    &:hover {
      > a.auto-anchor {
        display: block;
      }
    }
  }
}
