export { pathsToModuleNameMapper } from '../dist/config/paths-to-module-name-mapper'
export { createJestPreset } from '../dist/presets/create-jest-preset'
export { mocked } from '../dist/utils/testing'
