declare const rootDir: string
declare const cacheDir: string
declare const pkgDigestFile: string
declare const e2eSourceDir: string
declare const e2eRootDir: string
declare const e2eWorkDir: string
declare const e2eWorkTemplatesDir: string
declare const e2eTemplatesDir: string
declare const e2eWorkDirLink: string
declare const distDir: string
declare const testsRootDir: string
declare const e2eTestsDir: string
declare const projectsToRun: string[]

export {
  rootDir,
  cacheDir,
  pkgDigestFile,
  e2eSourceDir,
  e2eRootDir,
  e2eWorkDir,
  e2eWorkTemplatesDir,
  e2eTemplatesDir,
  e2eWorkDirLink,
  distDir,
  testsRootDir,
  e2eTestsDir,
  projectsToRun,
}
